####################################################################################
# In this script, data is simulated according to the baseline model. 
# Next, the full response mixture model and the baseline model are fit to the data
####################################################################################

library(MASS)
library(R2OpenBUGS)
set.seed()

N=2000
nit=20

output.full=matrix(,200,4125)
output.vdl=matrix(,200,4083)

for(iter in 1:80){
 
theta=matrix(rnorm(N))                               # similar as in the file for data simulation according to the full model
tau=.4*theta+sqrt(1-.4^2)*rnorm(N)
tau=tau*.4
  
item.par=read.table("baseline.itempar",he=T)

a=exp(item.par[,1])
b=item.par[,2]
ti=item.par[,3]
tr=exp(item.par[,4])

X=lnT=matrix(,N,nit)

ti=matrix(ti,N,nit,T)
tr=diag(tr)
 
b=matrix(b,N,nit,T)                                 # difficulty parameters

a=matrix(a,nit,1)                                   # discrimination parameters                                                

err=mvrnorm(N,rep(0,nit),tr)                        # similar as in the file for data simulation according to the full model
lrt=ti - tau%x%matrix(1,1,nit) + err

dev=theta%x%t(a) - b

p = plogis(dev) 

data=((p>(matrix(runif(N*nit),N,nit)))*1)

data->x

################################################################
##### FIT RESP MIX USING R Package R2OpenBUGS and OpenBUGS #####
################################################################

item.par=cbind(                                                # similar as in the file for data simulation according to the full model
c(rep(0,nit)),
c(rep(0,nit)),
qlogis(apply(x,2,mean,na.rm=T)),
qlogis(apply(x,2,mean,na.rm=T)),
rep(1,nit),
rep(0,nit))

item.mu=apply(item.par,2,mean)
item.prec=diag(rep(.1,6))

inits1=list(list(item.par=item.par,theta=rnorm(N),speed=rnorm(N),cor=0,prec.s=1,lntime=0,marg=0,item.mu=item.mu,item.prec=item.prec))

dat=list(x=data,lrt=lrt,N=N,nit=nit,zeros=rep(0,6),R=diag(rep(0.01,6)))

nms=c("item.par","cor","prec.s","time","marg","speed","theta")


res=bugs(dat, inits1, nms, n.iter=8002, model.file="full.txt",
    n.chains=1, n.burnin=4002, n.thin=1, 
    debug=FALSE, DIC=TRUE, digits=5)

output.full[iter,1:4125]=colMeans(res$sims.matrix)

########################
#####    FIT VDL   #####
########################
rm(res)

item.par=cbind(                                             # similar as in the file for data simulation according to the full model
c(rep(0,nit)),
qlogis(apply(x,2,mean,na.rm=T)),
rep(1,nit),
rep(0,nit))

item.mu=apply(item.par,2,mean)
item.prec=diag(rep(.1,4))

inits1=list(list(item.par=item.par,theta=rnorm(N),speed=rnorm(N),cor=0,prec.s=1,item.mu=item.mu,item.prec=item.prec))

dat=list(x=data,lrt=lrt,N=N,nit=nit,zeros=rep(0,4),R=diag(rep(0.01,4)))

nms=c("item.par","cor","prec.s","speed","theta")


res=bugs(dat, inits1, nms, n.iter=8002, model.file="baseline.txt",
    n.chains=1, n.burnin=4002, n.thin=1, 
    debug=FALSE, DIC=TRUE, digits=5)

output.vdl[iter,1:4083]=colMeans(res$sims.matrix)

}   
    


     


